@@ -1,5 +1,6 @@
 import os
 import random as rand
+import time
 from pathlib import Path
 
 import numpy as np
@@ -11,7 +12,7 @@ from vimms.ChemicalSamplers import UniformMZFormulaSampler, UniformRTAndIntensit
     MZMLFormulaSampler, MZMLRTandIntensitySampler, MZMLChromatogramSampler
 from vimms.Chemicals import ChemicalMixtureCreator, ChemicalMixtureFromMZML
 from vimms.Common import load_obj, set_log_level_warning, set_log_level_debug, \
-    ADDUCT_DICT_POS_MH, ScanParameters
+    ADDUCT_DICT_POS_MH, ScanParameters, set_log_level_info
 from vimms.Roi import RoiBuilderParams
 
 # define some useful constants
@@ -30,7 +31,8 @@ MIN_RT = RT_RANGE[0][0]
 MAX_RT = RT_RANGE[0][1]
 N_CHEMS = 10
 
-BEER_CHEMS = load_obj(Path(BASE_DIR, 'QCB_22May19_1.p'))
+BEER_CHEMS_PATH = Path(BASE_DIR, 'beer_compounds.p')
+BEER_CHEMS = load_obj(BEER_CHEMS_PATH)
 BEER_MIN_BOUND = 550
 BEER_MAX_BOUND = 650
 
@@ -48,22 +50,19 @@ def get_rt_bounds(dataset, centre):
 
 
 def run_environment(env):
-    # set the log level to WARNING so we don't see too many messages when environment is running
-    set_log_level_warning()
     # run the simulation
-    logger.info('Running simulation')
+    start_time = time.time()
     env.run()
-    logger.info('Done')
-    # set the log level back to DEBUG
-    set_log_level_debug()
+    logger.info('Done in %s seconds' % (time.time() - start_time))
 
 
-def check_mzML(env, out_dir, filename):
+def check_mzML(env, out_dir, filename, assert_size=None):
     out_file = os.path.join(out_dir, filename)
-    logger.info('Writing out mzML')
     env.write_mzML(out_dir, filename)
-    logger.info('Done')
     assert os.path.exists(out_file)
+    if assert_size is not None:
+        file_size = os.path.getsize(out_file)
+        assert file_size == assert_size
 
 
 def check_non_empty_MS1(controller):
@@ -124,6 +123,21 @@ def fragscan_dataset():
     return cm.sample(N_CHEMS, 2)
 
 
+@pytest.fixture(scope="module")
+def large_fragscan_dataset():
+    np.random.seed(0)
+    rand.seed(0)
+    min_mz = MZ_RANGE[0][0]
+    max_mz = MZ_RANGE[0][1]
+    min_rt = RT_RANGE[0][0]
+    max_rt = RT_RANGE[0][1]
+    um = UniformMZFormulaSampler(min_mz=min_mz, max_mz=max_mz)
+    ri = UniformRTAndIntensitySampler(min_rt=min_rt, max_rt=max_rt)
+    cs = GaussianChromatogramSampler(sigma=100)
+    cm = ChemicalMixtureCreator(um, rt_and_intensity_sampler=ri, chromatogram_sampler=cs)
+    return cm.sample(N_CHEMS*100, 2)
+
+
 @pytest.fixture(scope="module")
 def simple_dataset():
     np.random.seed(0)
