@@ -10,7 +10,8 @@ from tests.conftest import HMDB, MGF_FILE, MZML_FILE, OUT_DIR, check_mzML, check
 from vimms.ChemicalSamplers import UniformRTAndIntensitySampler, DatabaseFormulaSampler, \
     UniformMZFormulaSampler, CRPMS2Sampler, MGFMS2Sampler, MZMLMS2Sampler, ExactMatchMS2Sampler, \
     MZMLRTandIntensitySampler, MZMLFormulaSampler, MZMLChromatogramSampler, MzMLScanTimeSampler
-from vimms.Chemicals import ChemicalMixtureCreator, MultipleMixtureCreator, ChemicalMixtureFromMZML
+from vimms.Chemicals import ChemicalMixtureCreator, MultipleMixtureCreator, \
+    ChemicalMixtureFromMZML
 from vimms.Common import ADDUCT_DICT_POS_MH, POSITIVE, set_log_level_warning, \
     DEFAULT_SCAN_TIME_DICT
 from vimms.Controller import SimpleMs1Controller, TopNController
@@ -210,7 +211,7 @@ class TestChemicalsFromMZML():
         ionisation_mode = POSITIVE
         controller = SimpleMs1Controller()
         ms = IndependentMassSpectrometer(ionisation_mode, chems_from_mzml)
-        env = Environment(ms, controller, 500, 600, progress_bar=True)
+        env = Environment(ms, controller, 500, 600, progress_bar=False)
         set_log_level_warning()
         env.run()
         filename = 'fullscan_from_mzml.mzML'
@@ -226,7 +227,7 @@ class TestChemicalsFromMZML():
         controller = TopNController(ionisation_mode, N, isolation_width, mz_tol, rt_tol,
                                     min_ms1_intensity)
         ms = IndependentMassSpectrometer(ionisation_mode, chems_from_mzml)
-        env = Environment(ms, controller, 500, 600, progress_bar=True)
+        env = Environment(ms, controller, 500, 600, progress_bar=False)
         set_log_level_warning()
         env.run()
         check_non_empty_MS2(controller)
@@ -237,7 +238,7 @@ class TestChemicalsFromMZML():
         ionisation_mode = POSITIVE
         controller = SimpleMs1Controller()
         ms = IndependentMassSpectrometer(ionisation_mode, chem_mz_rt_i_from_mzml)
-        env = Environment(ms, controller, 500, 600, progress_bar=True)
+        env = Environment(ms, controller, 500, 600, progress_bar=False)
         set_log_level_warning()
         env.run()
         filename = 'fullscan_mz_rt_i_from_mzml.mzML'
@@ -258,7 +259,7 @@ class TestScanTiming():
         # run simulation using default scan times
         ms = IndependentMassSpectrometer(ionisation_mode, chems_from_mzml,
                                          scan_duration=DEFAULT_SCAN_TIME_DICT)
-        env = Environment(ms, controller, 500, 600, progress_bar=True)
+        env = Environment(ms, controller, 500, 600, progress_bar=False)
         set_log_level_warning()
         env.run()
         filename = 'test_scan_time_default.mzML'
@@ -284,7 +285,7 @@ class TestScanTiming():
         ms = IndependentMassSpectrometer(ionisation_mode, chems, scan_duration=sd)
 
         # run simulation
-        env = Environment(ms, controller, 500, 600, progress_bar=True)
+        env = Environment(ms, controller, 500, 600, progress_bar=False)
         set_log_level_warning()
         env.run()
         filename = 'test_scan_time_from_mzml.mzML'
@@ -310,8 +311,8 @@ class TestScanTiming():
         ms = IndependentMassSpectrometer(ionisation_mode, chems, scan_duration=sd)
 
         # run simulation
-        env = Environment(ms, controller, 500, 600, progress_bar=True)
+        env = Environment(ms, controller, 500, 600, progress_bar=False)
         set_log_level_warning()
         env.run()
         filename = 'test_scan_time_mean_from_mzml.mzML'
-        check_mzML(env, OUT_DIR, filename)
+        check_mzML(env, OUT_DIR, filename)
\ No newline at end of file
