@@ -25,7 +25,7 @@ class TestMS1Controller:
         controller = SimpleMs1Controller()
 
         # create an environment to run both the mass spec and controller
-        env = Environment(mass_spec, controller, min_bound, max_bound, progress_bar=True)
+        env = Environment(mass_spec, controller, min_bound, max_bound, progress_bar=False)
         run_environment(env)
 
         # write simulated output to mzML file
@@ -40,7 +40,7 @@ class TestMS1Controller:
         controller = SimpleMs1Controller()
 
         # create an environment to run both the mass spec and controller
-        env = Environment(mass_spec, controller, BEER_MIN_BOUND, BEER_MAX_BOUND, progress_bar=True)
+        env = Environment(mass_spec, controller, BEER_MIN_BOUND, BEER_MAX_BOUND, progress_bar=False)
         run_environment(env)
 
         # write simulated output to mzML file
@@ -61,7 +61,7 @@ class TestMS1Controller:
         controller = SimpleMs1Controller(advanced_params=params)
 
         # create an environment to run both the mass spec and controller
-        env = Environment(mass_spec, controller, BEER_MIN_BOUND, BEER_MAX_BOUND, progress_bar=True)
+        env = Environment(mass_spec, controller, BEER_MIN_BOUND, BEER_MAX_BOUND, progress_bar=False)
         run_environment(env)
 
         # write simulated output to mzML file
