@@ -10,19 +10,24 @@ from abc import abstractmethod, ABCMeta
 from collections import defaultdict, namedtuple
 from decimal import Decimal
 from functools import reduce
-from operator import attrgetter
+from operator import or_, attrgetter
 
 import intervaltree
 import numpy as np
 from mass_spec_utils.data_import.mzmine import PickedBox
 
 class Point():
+    __slots__ = ("x", "y")
+
     def round(self, ndigits=8):
         self.x, self.y = round(self.x, ndigits), round(self.y, ndigits)
 
     def __init__(self, x, y): 
         self.x, self.y = float(x), float(y)
         self.round()
+        
+    def __iter__(self):
+        return iter((self.x, self.y))
 
     def __eq__(self, other_point): 
         return (
@@ -39,6 +44,7 @@ class Point():
 
 
 class Interval():
+    __slots__ = ("pt1", "pt2")
     errmsg = "Interval has to be flat on one of the dimensions!"
 
     def __init__(self, x1, x2, y1, y2):
@@ -67,13 +73,16 @@ class Interval():
 
 
 class Box():
+    __slots__ = ("id", "roi", "pt1", "pt2", "parents", "intensity")
+
     def __init__(self, x1, x2, y1, y2, 
                     parents=None, 
                     min_xwidth=0, 
                     min_ywidth=0, 
                     intensity=0, 
                     id=None, 
-                    roi=None):
+                    roi=None,
+                    round_digits=8):
                 
         self.id = id
         self.roi = roi
@@ -92,8 +101,7 @@ class Box():
             self.pt1.y = midpoint - min_ywidth / 2 
             self.pt2.y = midpoint + min_ywidth / 2
             
-        self.pt1.round()
-        self.pt2.round()
+        self.round(ndigits=round_digits)
 
     def __repr__(self):
         return "Box({}, {})".format(self.pt1, self.pt2)
@@ -108,8 +116,19 @@ class Box():
     def __hash__(self):
         return (self.pt1, self.pt2).__hash__()
         
-    def serialise_info(self):
-        return [self.min_rt, self.max_rt, self.min_mz, self.max_mz, self.intensity]
+    def round(self, ndigits=8):
+        self.pt1.round(ndigits=ndigits)
+        self.pt2.round(ndigits=ndigits)
+        
+    def serialise_info(self, minutes=False):
+        timescale = 60 if minutes else 1
+        return [
+            self.pt1.x / timescale, 
+            self.pt2.x / timescale, 
+            self.pt1.y, 
+            self.pt2.y, 
+            self.intensity
+        ]
 
     def area(self):
         return (self.pt2.x - self.pt1.x) * (self.pt2.y - self.pt1.y)
@@ -126,17 +145,19 @@ class Box():
                 roi=self.roi
                )
 
-    def shift(self, xshift=0, yshift=0):
+    def shift(self, xshift=0, yshift=0, round_digits=8):
         self.pt1.x += xshift
         self.pt2.x += xshift
         self.pt1.y += yshift
         self.pt2.y += yshift
-        self.pt1.round()
-        self.pt2.round()
+        self.round(ndigits=round_digits)
 
     def num_overlaps(self):
         return len(self.parents)
         
+    def to_box(self, min_rt_width=0.0, min_mz_width=0.0):
+        return self.copy()
+        
     @classmethod
     def from_pickedbox(cls, pbox):
         min_rt, max_rt = pbox.rt_range_in_seconds
@@ -196,6 +217,40 @@ class GenericBox(Box):
             and self.pt2.y >= other_box.pt2.y
         )
         
+    def combine_max(self, other_box):
+        return GenericBox(
+            min(self.pt1.x, other_box.pt1.x),
+            max(self.pt2.x, other_box.pt2.x),
+            min(self.pt1.y, other_box.pt1.y),
+            max(self.pt2.y, other_box.pt2.y),
+            parents=(self.parents + other_box.parents),
+            intensity=max(self.intensity, other_box.intensity),
+        )
+        
+    def apply_min_box_ppm(self, xwidth=None, ywidth=None, round_digits=8):
+        x1, y1 = self.pt1
+        x2, y2 = self.pt2
+    
+        if(not xwidth is None):
+            mid = (x1 + x2) / 2
+            dist = (xwidth / 1e6) * mid
+            if(dist > x2 - x1):
+                x1 = mid - dist / 2
+                x2 = mid + dist / 2
+        
+        if(not ywidth is None):
+            mid = (y1 + y2) / 2
+            dist = (ywidth / 1e6) * mid
+            if(dist > y2 - y1):
+                y1 = mid - dist / 2
+                y2 = mid + dist / 2
+            
+        new_box = self.copy()
+        new_box.pt1.x, new_box.pt1.y = x1, y1
+        new_box.pt2.x, new_box.pt2.y = x2, y2
+        new_box.round(ndigits=round_digits)
+        return new_box
+        
     def overlap_raw(self, other_box):
         if(not self.overlaps_with_box(other_box)): 
             return 0.0
@@ -291,10 +346,14 @@ class Grid(metaclass=ABCMeta):
         self.rt_box_size, self.mz_box_size = rt_box_size, mz_box_size
         self.box_area = float(Decimal(rt_box_size) * Decimal(mz_box_size))
 
-        self.rtboxes = range(0, int((self.max_rt - self.min_rt) /
-                                    rt_box_size) + 1)
-        self.mzboxes = range(0, int((self.max_mz - self.min_mz) /
-                                    mz_box_size) + 1)
+        self.rtboxes = range(
+            0, 
+            int((self.max_rt - self.min_rt) / rt_box_size) + 2
+        )
+        self.mzboxes = range(
+            0, 
+            int((self.max_mz - self.min_mz) / mz_box_size) + 2
+        )
         self.boxes = self.init_boxes(self.rtboxes, self.mzboxes)
 
     def get_box_ranges(self, box):
@@ -384,14 +443,13 @@ class LocatorGrid(Grid):
     def get_boxes(self, box):
         rt_box_range, mz_box_range, _ = self.get_box_ranges(box)
         boxes = set()
-        for row in self.boxes[rt_box_range[0]:rt_box_range[1],
-                   mz_box_range[0]:mz_box_range[1]]:
+        for row in self.boxes[rt_box_range[0]:rt_box_range[1], mz_box_range[0]:mz_box_range[1]]:
             for s in row:
                 boxes |= s
         return boxes
 
     def get_all_boxes(self):
-        return reduce(lambda s1, s2: s1 | s2, (s for row in self.boxes for s in row))
+        return reduce(or_, (s for row in self.boxes for s in row), set())
 
     def register_box(self, box):
         rt_box_range, mz_box_range, _ = self.get_box_ranges(box)
@@ -417,6 +475,9 @@ class LineSweeper():
     def get_all_boxes(self):
         return self.all_boxes
         
+    def get_active_boxes(self):
+        return self.active
+        
     def _add_active(self, new_ptr, current_loc):
         for b in itertools.islice(self.all_boxes, self.active_pointer, new_ptr):
             self.active.append(b)
@@ -428,7 +489,23 @@ class LineSweeper():
         b = self.active.pop()
         self.was_active.append(b)
         if(not b in self.removed):
-            self.active_intervals.removei(b.pt1.y, b.pt2.y + 1E-12, b)
+            try:
+                self.active_intervals.removei(b.pt1.y, b.pt2.y + 1E-12, b)
+            except KeyError:
+                inv = intervaltree.Interval(b.pt1.y, b.pt2.y + 1E-12, b)
+                if(inv in self.active_intervals): #hack for rare failure mode for intervaltree
+                    new_tree = intervaltree.IntervalTree()
+                    for other in self.active_intervals:
+                        if(inv != other): new_tree.add(other)
+                    errstr = f"""
+                        Failure when trying to error-correct IntervalTree failing remove
+                        len(old_tree) == {self.active_intervals}
+                        len(new_tree) == {new_tree}
+                    """
+                    assert len(self.active_intervals) == len(new_tree) + 1, errstr
+                    self.active_intervals = new_tree
+                else:
+                    raise
             self.removed.add(b)
     
     def set_active_boxes(self, current_loc):
@@ -446,7 +523,7 @@ class LineSweeper():
         self._add_active(new_ptr, current_loc)
         
         self.was_active = []
-        while(self.active != [] and self.active[-1].pt2.x <= current_loc):
+        while(self.active != [] and self.active[-1].pt2.x < current_loc):
             self._remove_active()
         self.previous_loc = self.current_loc
         self.current_loc = current_loc
@@ -768,7 +845,14 @@ class BoxExact(BoxGeometry):
         
             
 class BoxGrid(BoxExact): 
-    def __init__(self, min_rt, max_rt, rt_box_size, min_mz, max_mz, mz_box_size):
+    def __init__(self, 
+                 min_rt=0, 
+                 max_rt=1440, 
+                 rt_box_size=50, 
+                 min_mz=0, 
+                 max_mz=2000, 
+                 mz_box_size=1):
+        
         self.grid = LocatorGrid(min_rt, max_rt, rt_box_size, min_mz, max_mz, mz_box_size)
         
     def get_all_boxes(self):
@@ -777,7 +861,7 @@ class BoxGrid(BoxExact):
     def point_in_box(self, pt):
         box = GenericBox(pt.x, pt.x, pt.y, pt.y)
         return any(
-            box.contains_point(pt) for b in self.grid.get_boxes(box)
+            b.contains_point(pt) for b in self.grid.get_boxes(box)
         )
         
     def point_in_which_boxes(self, pt):
@@ -804,6 +888,32 @@ class BoxGrid(BoxExact):
         )
         
     def register_boxes(self, boxes):
+        boxes = list(boxes)
+        min_rt, max_rt = self.grid.min_rt, self.grid.max_rt
+        min_mz, max_mz = self.grid.min_mz, self.grid.max_mz
+        rt_box_size, mz_box_size = self.grid.rt_box_size, self.grid.mz_box_size
+        
+        resize = any(
+            min_rt > b.pt1.x
+            or max_rt < b.pt2.x
+            or min_mz > b.pt1.y
+            or max_mz < b.pt2.y
+            for b in boxes
+        )
+        
+        if(resize):
+            new_grid = LocatorGrid(
+                min(min_rt, min(b.pt1.x for b in boxes) - rt_box_size), 
+                max(max_rt, max(b.pt2.x for b in boxes) + rt_box_size), 
+                rt_box_size,
+                min(min_mz, min(b.pt1.y for b in boxes) - mz_box_size), 
+                max(max_mz, max(b.pt2.y for b in boxes) + mz_box_size), 
+                mz_box_size
+            )
+            for b in self.grid.get_all_boxes():
+                new_grid.register_box(b)
+            self.grid = new_grid
+        
         for b in boxes:
             self.grid.register_box(b)
         
