@@ -3,7 +3,10 @@ Provides implementation of Chemicals objects that are used as input
 to the simulation.
 """
 import copy
-from abc import ABC, abstractmethod
+import itertools
+import pickle
+from abc import ABCMeta, abstractmethod
+from collections import deque
 
 import numpy as np
 import scipy
@@ -13,9 +16,9 @@ from loguru import logger
 from vimms.ChemicalSamplers import UniformRTAndIntensitySampler, \
     GaussianChromatogramSampler, UniformMS2Sampler
 from vimms.Chromatograms import EmpiricalChromatogram
-from vimms.Common import POS_TRANSFORMATIONS, Formula, DummyFormula, \
+from vimms.Common import Formula, DummyFormula, \
     PROTON_MASS, POSITIVE, NEGATIVE, C12_PROPORTION, \
-    C13_MZ_DIFF, C, MONO, C13
+    C13_MZ_DIFF, C, MONO, C13, load_obj, ADDUCT_NAMES_POS, ADDUCT_NAMES_NEG
 from vimms.Noise import GaussianPeakNoise
 from vimms.Roi import make_roi, RoiBuilderParams
 
@@ -145,11 +148,17 @@ class Adducts():
             adduct_prior_dict: custom adduct dictionary, if any
         """
         if adduct_prior_dict is None:
-            self.adduct_names = {POSITIVE: list(POS_TRANSFORMATIONS.keys())}
+            self.adduct_names = {
+                POSITIVE: ADDUCT_NAMES_POS,
+                NEGATIVE: ADDUCT_NAMES_NEG
+            }
             self.adduct_prior = {
-                POSITIVE: np.ones(len(self.adduct_names[POSITIVE])) * 0.1}
-            self.adduct_prior[POSITIVE][
-                0] = 1.0  # give more weight to the first one, i.e. M+H
+                POSITIVE: np.ones(len(self.adduct_names[POSITIVE])) * 0.1,
+                NEGATIVE: np.ones(len(self.adduct_names[NEGATIVE])) * 0.1
+            }
+            # give more weight to the first one, i.e. M+H
+            self.adduct_prior[POSITIVE][0] = 1.0
+            self.adduct_prior[NEGATIVE][0] = 1.0
         else:
             assert POSITIVE in adduct_prior_dict or \
                    NEGATIVE in adduct_prior_dict
@@ -205,7 +214,7 @@ class Adducts():
         return self.adduct_names
 
 
-class BaseChemical(ABC):
+class BaseChemical(metaclass=ABCMeta):
     """
     The base class for Chemical objects across all MS levels.
     Chemicals at MS level = 1 is special and should be instantiated as either Known
@@ -213,6 +222,11 @@ class BaseChemical(ABC):
     For other MS levels, please use the MSN class.
     """
 
+    __slots__ = (
+        "ms_level",
+        "children"
+    )
+
     def __init__(self, ms_level, children):
         """
         Defines a base chemical object
@@ -230,12 +244,20 @@ class Chemical(BaseChemical):
     Should be realised as either Known or Unknown chemicals.
     """
 
+    __slots__ = (
+        "rt",
+        "max_intensity",
+        "chromatogram",
+        "mz_diff",
+        "base_chemical",
+    )
+
     def __init__(self, rt, max_intensity, chromatogram, children,
                  base_chemical):
         """
         Create a Chemical object
         Args:
-            rt: the RT value of this chemical
+            rt: the starting RT value of this chemical
             max_intensity: the maximum intensity of this chemical
             chromatogram: the chromatogram of this chemical
             children: any children of this chemical
@@ -259,6 +281,12 @@ class Chemical(BaseChemical):
 
         return self.rt + self.chromatogram.get_apex_rt()
 
+    def get_min_rt(self):
+        return self.chromatogram.min_rt + self.rt
+
+    def get_max_rt(self):
+        return self.chromatogram.max_rt + self.rt
+
     def get_original_parent(self):
         """
         Get the original base chemical in a recursive manner.
@@ -269,24 +297,6 @@ class Chemical(BaseChemical):
         return self if self.base_chemical is None else \
             self.base_chemical.get_original_parent()
 
-    @abstractmethod
-    def get_key(self):
-        """
-        Turns a chemical object into some sensible keys for comparison
-
-        Returns: keys for comparisons in __eq__ and __hash__
-
-        """
-        pass
-
-    def __eq__(self, other):
-        if type(other) is type(self):
-            return self.get_key() == other.get_key()
-        return False
-
-    def __hash__(self):
-        return hash(self.get_key())
-
 
 class UnknownChemical(Chemical):
     """
@@ -295,6 +305,12 @@ class UnknownChemical(Chemical):
     from an existing mzML file.
     """
 
+    __slots__ = (
+        "isotopes",
+        "adducts",
+        "mass"
+    )
+
     def __init__(self, mz, rt, max_intensity, chromatogram, children=None,
                  base_chemical=None):
         """
@@ -303,7 +319,7 @@ class UnknownChemical(Chemical):
         Args:
             mz: the m/z value of this chemical. Unlike [vimms.Chemicals.KnownChemical][] here we
                 know the m/z value but do not known the formula that generates this chemical.
-            rt: the RT value of this chemical
+            rt: the starting RT value of this chemical
             max_intensity: the maximum intensity of this chemical
             chromatogram: the chromatogram of this chemical
             children: any children of this chemical
@@ -315,9 +331,6 @@ class UnknownChemical(Chemical):
         self.adducts = {POSITIVE: [("M+H", 1)], NEGATIVE: [("M-H", 1)]}
         self.mass = mz
 
-    def get_key(self):
-        return tuple(self.isotopes), self.rt, self.max_intensity
-
     def __repr__(self):
         return 'UnknownChemical mz=%.4f rt=%.2f max_intensity=%.2f' % (
             self.isotopes[0][0], self.rt, self.max_intensity)
@@ -340,7 +353,7 @@ class KnownChemical(Chemical):
             formula: the formula of this chemical object.
             isotopes: the isotope of this chemical object
             adducts: the adduct of this chemical object
-            rt: the retention time value of this chemical object
+            rt: the starting retention time value of this chemical object
             max_intensity: the maximum intensity value in the chromatogram
             chromatogram: the chromatogram of the chemical
             children: any children of the chemical
@@ -362,19 +375,204 @@ class KnownChemical(Chemical):
         self.mass = self.formula.mass
         self.database_accession = database_accession
 
-    def get_key(self):
-        return tuple(self.formula.formula_string), self.rt
-
     def __repr__(self):
         return 'KnownChemical - %r rt=%.2f max_intensity=%.2f' % (
             self.formula.formula_string, self.rt, self.max_intensity)
 
 
+class ChemSet():
+    def reset(self):
+        self.rt = 0
+        self.current = []
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        pass
+
+    @classmethod
+    def to_chemset(cls, chems, filepath=None, fast=False):
+        if type(chems) == MemoryChems or \
+                type(chems) == FileChems or \
+                type(chems) == FastMemoryChems:
+            return chems
+
+        if filepath is None:
+            if fast:
+                return FastMemoryChems(chems)
+            else:
+                return MemoryChems(chems)
+        else:
+            return FileChems(filepath)
+
+    @staticmethod
+    def dump_chems(chems, filepath):
+        key = Chemical.get_min_rt
+        srted = sorted(chems, key=key)
+        grouped = itertools.groupby(srted, key=lambda ch: round(Chemical.get_min_rt(ch), 1))
+        with open(filepath, "wb") as f:
+            for k, group in grouped:
+                pickle.dump(list(group), f, protocol=pickle.HIGHEST_PROTOCOL)
+
+    def _update(self, rt, chems):
+        key = Chemical.get_max_rt
+        self.current.extend(chems)
+        self.current.sort(key=key, reverse=True)
+        while (len(self.current) > 0 and Chemical.get_max_rt(self.current[-1]) < rt):
+            self.current.pop()
+        self.rt = rt
+
+    @abstractmethod
+    def next_chems(self, rt):
+        pass
+
+
+class MemoryChems(ChemSet):
+    def __init__(self, local_chems):
+        logger.debug('MemoryChems initialised')
+        key = Chemical.get_min_rt
+        self.local_chems = sorted(local_chems, key=key)
+        self.reset()
+
+    def reset(self):
+        self.pos = 0
+        super().reset()
+
+    def __iter__(self):
+        return iter(self.local_chems)
+
+    @classmethod
+    def from_chems(cls, chems):
+        if (type(chems) == cls):
+            return chems
+        return cls(chems)
+
+    @classmethod
+    def from_path(cls, filepath):
+        chems = []
+        with open(filepath, "rb") as f:
+            try:
+                while (True):
+                    chems.extend(pickle.load(f))
+            except EOFError:
+                pass
+        return cls(chems)
+
+    def next_chems(self, rt):
+        if (rt < self.rt): self.reset()
+        new_pos = self.pos
+        while (new_pos < len(self.local_chems)
+               and Chemical.get_min_rt(self.local_chems[new_pos]) <= rt):
+            new_pos += 1
+        self._update(rt, itertools.islice(self.local_chems, self.pos, new_pos))
+        self.pos = new_pos
+        return np.array(list(reversed(self.current)))
+
+
+# TODO: slightly faster than MemoryChems, but can be made faster with intervaltree.
+class FastMemoryChems(MemoryChems):
+
+    def __init__(self, local_chems):
+        logger.debug('FastMemoryChems initialised')
+        super().reset()
+        self.local_chems = np.array(local_chems)
+
+        chem_rts = np.array([chem.rt for chem in self.local_chems])
+        self.chrom_min_rts = np.array(
+            [chem.chromatogram.min_rt for chem in self.local_chems]) + chem_rts
+        self.chrom_max_rts = np.array(
+            [chem.chromatogram.max_rt for chem in self.local_chems]) + chem_rts
+
+    def next_chems(self, rt):
+        idx = np.where((self.chrom_min_rts < rt) & (rt < self.chrom_max_rts))[0]
+        return self.local_chems[idx]
+
+
+class FileChems(ChemSet):
+    def __init__(self, filepath):
+        logger.debug('FileChems initialised')
+        self.filepath = filepath
+        self.f = None
+        self.reset()
+
+    def reset(self):
+        if (not self.f is None):
+            self.f.close()
+            self.f = None
+        self.pending = deque()
+        self.finished = False
+        super().reset()
+
+    def __iter__(self):
+        with open(self.filepath, "rb") as f:
+            try:
+                while (True):
+                    for ch in pickle.load(f):
+                        yield ch
+            except EOFError:
+                pass
+
+    def __exit__(self, type, value, traceback):
+        if (not self.f is None):
+            self.f.close()
+
+    @classmethod
+    def from_path(cls, filepath, chems=None):
+        if (type(chems) == cls):
+            return chems
+
+        if (not chems is None):
+            cls.dump_chems(chems, filepath)
+
+        return cls(filepath)
+
+    def next_chems(self, rt):
+        if (rt < self.rt): self.reset()
+        if (self.finished):
+            self._update(rt, [])
+            return np.array(list(reversed(self.current)))
+
+        if (self.f is None):
+            self.f = open(self.filepath, "rb")
+
+        try:
+            while (not self.finished and
+                   (len(self.pending) == 0 or Chemical.get_min_rt(self.pending[-1]) <= rt)):
+                try:
+                    new_chems = pickle.load(self.f)
+                except pickle.UnpicklingError:
+
+                    # failed to unpickle chems previously saved using save_obj
+                    # try to load again using load_obj
+                    key = Chemical.get_min_rt
+                    new_chems = sorted(load_obj(self.filepath), key=key)  # important to sort
+
+                self.pending.extend(new_chems)
+        except EOFError:
+            self.finished = True
+            self.f.close()
+
+        new = []
+        while (len(self.pending) > 0 and Chemical.get_min_rt(self.pending[0]) <= rt):
+            new.append(self.pending.popleft())
+
+        self._update(rt, new)
+        return np.array(list(reversed(self.current)))
+
+
 class MSN(BaseChemical):
     """
     A chemical that represents an MS2+ fragment.
     """
 
+    __slots__ = (
+        "isotopes",
+        "prop_ms2_mass",
+        "parent_mass_prop",
+        "parent"
+    )
+
     def __init__(self, mz, ms_level, prop_ms2_mass, parent_mass_prop,
                  children=None, parent=None):
         """
@@ -389,9 +587,7 @@ class MSN(BaseChemical):
             parent: parent MS1 peak
         """
         super().__init__(ms_level, children)
-
         self.isotopes = [(mz, None, "MSN")]
-        self.ms_level = ms_level
         self.prop_ms2_mass = prop_ms2_mass
         self.parent_mass_prop = parent_mass_prop
         self.parent = parent
@@ -661,7 +857,7 @@ class ChemicalMixtureFromMZML():
         chemicals = []
         for roi_idx in rois_to_use:
             r = self.good_rois[roi_idx]
-            mz = r.get_mean_mz()
+            mz = r.mean_mz
             if source_polarity == POSITIVE:
                 mz -= PROTON_MASS
             elif source_polarity == NEGATIVE:
