@@ -228,9 +228,6 @@ class Controller(ABC):
         Returns: a list of new [vimms.Common.ScanParameters][] describing what to do next.
 
         """
-        logger.debug('tasks to be sent = %d' % (current_size))
-        logger.debug('tasks sent but not received = %d' % (pending_size))
-
         # record every scan that we've received
         self.scans[scan.ms_level].append(scan)
 
@@ -243,17 +240,11 @@ class Controller(ABC):
         # have been sent and processed AND this ms1 scan is a custom scan
         # we'd sent before (not a method scan) then store it for
         # fragmentation next time
-        logger.debug(
-            'scan.scan_id = %d, self.next_processed_scan_id = %d' % (
-                scan.scan_id, self.next_processed_scan_id))
+
         if scan.scan_id == self.next_processed_scan_id:
             self.scan_to_process = scan
-            logger.debug('Next processed scan %d has arrived' %
-                         self.next_processed_scan_id)
         else:
             self.scan_to_process = None
-        logger.debug('scan_to_process = %s' % self.scan_to_process)
-        logger.debug('scan.scan_params = %s' % scan.scan_params)
 
         # implemented by subclass
         if self.scan_to_process is not None:
@@ -362,3 +353,64 @@ class Controller(ABC):
 
         """
         pass
+
+        
+class WrapperController(Controller):
+    """
+    Template for controller which wraps behaviour of at least one other 
+    controller.
+    """
+    
+    def __init__(self):
+        self.__dict__.update(self.controller.__dict__)
+        
+    def get_ms1_scan_params(self, metadata=None):
+        val = self.controller.get_ms1_scan_params(metadata=None)
+        self.__dict__.update(self.controller.__dict__)
+        return val
+        
+    def get_ms2_scan_params(self, mz, intensity, precursor_scan_id, isolation_width, 
+                            mz_tol, rt_tol, metadata=None):
+                            
+        val = self.controller.get_ms2_scan_params(
+            mz, intensity, precursor_scan_id, isolation_width, mz_tol, rt_tol, metadata=None
+        )
+        self.__dict__.update(self.controller.__dict__)
+        return val
+    
+    def get_initial_tasks(self):
+        val = self.controller.get_initial_tasks()
+        self.__dict__.update(self.controller.__dict__)
+        return val
+        
+    def get_initial_scan_params(self):
+        val = self.controller.get_initial_scan_params()
+        self.__dict__.update(self.controller.__dict__)
+        return val
+        
+    def set_environment(self, env):
+        val = self.controller.set_environment(env)
+        self.__dict__.update(self.controller.__dict__)
+        return val
+        
+    def handle_scan(self, scan, current_size, pending_size):
+        val = self.controller.handle_scan(scan, current_size, pending_size)
+        self.__dict__.update(self.controller.__dict__)
+        return val
+        
+    def update_state_after_scan(self, last_scan):
+        self.controller.update_state_after_scan(last_scan)
+        self.__dict__.update(self.controller.__dict__)
+        
+    def _process_scan(self, scan):
+        val = self.controller._process_scan(scan)
+        self.__dict__.update(self.controller.__dict__)
+        return val
+        
+    def dump_scans(self, output_method):
+        self.controller.dump_scans(output_method)
+        self.__dict__.update(self.controller.__dict__)
+        
+    def after_injection_cleanup(self):
+        self.controller.after_injection_cleanup()
+        self.__dict__.update(self.controller.__dict__)
