@@ -7,13 +7,14 @@ import itertools
 import math
 import os
 import subprocess
+from abc import ABC, ABCMeta, abstractmethod
 
 import numpy as np
 from loguru import logger
 
 from vimms.Common import INITIAL_SCAN_ID, get_default_scan_params, \
     get_dda_scan_param, DEFAULT_ISOLATION_WIDTH
-from vimms.Controller.base import Controller
+from vimms.Controller.base import Controller, WrapperController
 
 
 class FixedScansController(Controller):
@@ -77,6 +78,24 @@ class FixedScansController(Controller):
 
     def _process_scan(self, scan):
         pass
+        
+
+class DsDAController(WrapperController):
+    def __init__(self, dsda_state, mzml_name):
+        self.dsda_state = dsda_state
+        self.mzml_name = mzml_name
+        
+        if(dsda_state.file_num == 0):
+            self.controller = self.dsda_state.get_base_controller()
+        else:
+            schedule_params = self.dsda_state.get_scan_params()
+            self.controller = FixedScansController(schedule=schedule_params)
+            
+        print(self.controller)
+        super().__init__()
+
+    def after_injection_cleanup(self):
+        self.dsda_state.register_mzml(self.mzml_name)
 
 
 class MS2PlannerController(FixedScansController):
@@ -127,8 +146,7 @@ class MS2PlannerController(FixedScansController):
                         sp[intensity]
                     ])
 
-        out_headers = ["Mass [m/z]", "retention_time", "charge", "Blank",
-                       "Sample"]
+        out_headers = ["Mass [m/z]", "retention_time", "charge", "Blank", "Sample"]
         with open(outpath, "w+") as f:
             f.write(",".join(out_headers) + "\n")
             for r in records:
@@ -292,9 +310,9 @@ class MS2PlannerController(FixedScansController):
             ]
         )
         schedules = [
-            MS2PlannerController.sched_dict2params(sch, scan_duration_dict) for
-            sch in
-            MS2PlannerController.parse_ms2planner(out_file)]
+            MS2PlannerController.sched_dict2params(sch, scan_duration_dict) 
+            for sch in MS2PlannerController.parse_ms2planner(out_file)
+        ]
         with open(os.path.join(os.path.dirname(out_file), "scan_params.txt"),
                   "w+") as f:
             for i, schedule in enumerate(schedules):
@@ -302,8 +320,10 @@ class MS2PlannerController(FixedScansController):
                 f.write("".join(
                     f"SCAN {j}: {scan}\n\n" for j, scan in enumerate(schedule))
                 )
-        return [MS2PlannerController(schedule=schedule, params=params) for
-                schedule in schedules]
+        return [
+            MS2PlannerController(schedule=schedule, params=params) 
+            for schedule in schedules
+        ]
 
 
 class MatchingController(FixedScansController):
@@ -311,11 +331,12 @@ class MatchingController(FixedScansController):
     A pre-scheduled controller that performs maximum matching to obtain the largest
     coverage
     """
-    @staticmethod
-    def from_matching(matching, isolation_width, params=None):
-        return [MatchingController(schedule=schedule, params=params) for
-                schedule in
-                matching.make_schedules(isolation_width)]
+    @classmethod
+    def from_matching(cls, matching, isolation_width, advanced_params=None):
+        return [
+            MatchingController(schedule=schedule, advanced_params=advanced_params) 
+            for schedule in matching.make_schedules(isolation_width)
+        ]
 
 
 class MultiIsolationController(Controller):
