@@ -1,18 +1,21 @@
 import math
+from collections import defaultdict
 
 import numpy as np
-import scipy
 from events import Events
-from loguru import logger
 
+from vimms.Chemicals import ChemSet
 from vimms.Common import (
-    adduct_transformation, DEFAULT_SCAN_TIME_DICT,
-    INITIAL_SCAN_ID, ScanParameters
+    DEFAULT_SCAN_TIME_DICT,
+    INITIAL_SCAN_ID, ScanParameters, ADDUCT_TERMS, PEAKS_MZ_IDX, PEAKS_INTENSITY_IDX,
+    PEAKS_MS1_INTENSITY_IDX, PEAKS_WHICH_ISOTOPE_IDX, PEAKS_WHICH_ADDUCT_IDX
 )
+from vimms.MassSpecUtils import get_mz_ms1, get_mz_msn, \
+    generate_chem_ms1_peaks_for_ms2, generate_chem_ms1_peaks_for_ms1, ChemDataCollector
 from vimms.Noise import NoPeakNoise
 
 
-class Peak():
+class ScanEventPeak():
     """
     A class to represent an empirical or sampled scan-level peak object
     """
@@ -37,7 +40,7 @@ class Peak():
             self.mz, self.rt, self.intensity, self.ms_level)
 
     def __eq__(self, other):
-        if not isinstance(other, Peak):
+        if not isinstance(other, ScanEventPeak):
             # don't attempt to compare against unrelated types
             return NotImplemented
 
@@ -85,16 +88,16 @@ class Scan():
         self.scan_params = scan_params
         self.parent = parent
         self.fragevent = fragevent
-        
+
     @classmethod
     def from_mzmlscan(self, scan):
         mzs, intensities = zip(*scan.peaks)
         return Scan(
-            scan_id = scan.scan_no,
-            mzs = np.array(mzs),
-            intensities = np.array(intensities),
-            ms_level = scan.ms_level,
-            rt = scan.rt_in_seconds
+            scan_id=scan.scan_no,
+            mzs=np.array(mzs),
+            intensities=np.array(intensities),
+            ms_level=scan.ms_level,
+            rt=scan.rt_in_seconds
         )
 
     def __repr__(self):
@@ -287,7 +290,8 @@ class IndependentMassSpectrometer():
                  intensity_noise=None, spike_noise=None,
                  isolation_transition_window='rectangular',
                  isolation_transition_window_params=None,
-                 scan_duration=DEFAULT_SCAN_TIME_DICT, task_manager=None):
+                 scan_duration=DEFAULT_SCAN_TIME_DICT, task_manager=None,
+                 skip_ms2_spectra_generation=False):
         """
         Creates a mass spec object.
 
@@ -330,15 +334,9 @@ class IndependentMassSpectrometer():
         }
 
         # the list of all chemicals in the dataset
-        self.chemicals = chemicals
+        self.chemicals = ChemSet.to_chemset(chemicals, fast=True)
         self.ionisation_mode = ionisation_mode
-
-        # stores the chromatograms start and end rt for quick retrieval
-        chem_rts = np.array([chem.rt for chem in self.chemicals])
-        self.chrom_min_rts = np.array(
-            [chem.chromatogram.min_rt for chem in self.chemicals]) + chem_rts
-        self.chrom_max_rts = np.array(
-            [chem.chromatogram.max_rt for chem in self.chemicals]) + chem_rts
+        self.chem_data_collector = ChemDataCollector(self.ionisation_mode)
 
         # whether to add noise to the generated peaks, the default is no noise
         self.mz_noise = mz_noise
@@ -355,6 +353,7 @@ class IndependentMassSpectrometer():
         self.isolation_transition_window_params = isolation_transition_window_params  # noqa
 
         self.scan_duration_dict = scan_duration
+        self.skip_ms2_spectra_generation = skip_ms2_spectra_generation
 
     ###########################################################################
     # Public methods
@@ -380,14 +379,12 @@ class IndependentMassSpectrometer():
             # This could return an empty list if there's nothing there.
             params_list = self.get_params()
         else:
-            logger.debug('Got an initial task from controller')
             params_list = [params]  # initial param passed from the controller
 
         # Send params away. In the simulated case, no sending actually occurs,
         # instead we just track these params we've sent by adding them to
         # self.environment.pending_tasks
         if len(params_list) > 0:
-            logger.debug('new tasks ready to send = %d' % len(params_list))
             self.send_params(params_list)
 
         # pick up the last param that has been sent and generate a new scan
@@ -440,7 +437,6 @@ class IndependentMassSpectrometer():
 
         """
         params_list = self.task_manager.to_send()
-        logger.debug('Selected %d tasks to send' % (len(params_list)))
         return params_list
 
     def send_params(self, params_list):
@@ -458,7 +454,6 @@ class IndependentMassSpectrometer():
 
         """
         self.task_manager.add_pending(params_list)
-        logger.debug('Successfully sent %d tasks' % (len(params_list)))
 
     def fire_event(self, event_name, arg=None):
         """
@@ -530,7 +525,6 @@ class IndependentMassSpectrometer():
         Returns: None
 
         """
-        logger.debug('Unregistering event handlers')
         self.clear_events()
 
     ###########################################################################
@@ -571,8 +565,6 @@ class IndependentMassSpectrometer():
                                                         next_level)
 
         self.time += current_scan_duration
-        logger.debug(
-            'scan_duration=%f time %f' % (current_scan_duration, self.time))
         return current_scan_duration
 
     ###########################################################################
@@ -595,32 +587,13 @@ class IndependentMassSpectrometer():
 
         min_measurement_mz = params.get(ScanParameters.FIRST_MASS)
         max_measurement_mz = params.get(ScanParameters.LAST_MASS)
-
         ms1_source_collision_energy = params.get(
             ScanParameters.SOURCE_CID_ENERGY)
-
-        scan_mzs = []  # all the mzs values in this scan
-        scan_intensities = []  # all the intensity values in this scan
         ms_level = params.get(ScanParameters.MS_LEVEL)
-        if ms_level == 1:  # if ms1 then we scan the whole range of m/z
-            isolation_windows = params.get(ScanParameters.ISOLATION_WINDOWS)
-            if isolation_windows is None:
-                isolation_windows = [
-                    [(min_measurement_mz, max_measurement_mz)]]
-            if not isolation_windows[0][0][0] == min_measurement_mz or not \
-                    isolation_windows[0][0][
-                        1] == max_measurement_mz:
-                logger.warning(
-                    "MS1 scan: MS1 isolation window and measurement "
-                    "mz range mismatch")
 
-        else:
-            # if ms2 then we check if the isolation window parameter
-            # is specified
-            # isolation_windows = params.get(ScanParameters.ISOLATION_WINDOWS)
-            # if not then we compute from the precursor mz and isolation width
-            # if isolation_windows is None:
-            isolation_windows = params.compute_isolation_windows()
+        # compute isolation window, depending on whether it's MS1 or MS2 scan
+        isolation_windows = self._get_isolation_windows(max_measurement_mz, min_measurement_mz,
+                                                        ms_level, params)
 
         # if the scan id is specified in the params, use it
         # otherwise use the one that has been incremented from the previous one
@@ -628,73 +601,40 @@ class IndependentMassSpectrometer():
         if scan_id is None:
             scan_id = self.idx
 
-        # for all chemicals that come out from the column
-        # coupled to the mass spec
-        idx = self._get_chem_indices(scan_time)
-
         # the following is to ensure we generate fragment data when
         # we have a collision energe >0
         use_ms_level = ms_level
         if ms_level == 1 and ms1_source_collision_energy > 0:
             use_ms_level = 2
 
-        frag_events = []
-        for i in idx:
-            chemical = self.chemicals[i]
-            # mzs is a list of (mz, intensity) for the different
-            # adduct/isotopes combinations of a chemical
-            mzs = self._get_all_mz_peaks(chemical, scan_time, use_ms_level,
-                                         isolation_windows)
-            peaks = []
-            peaks_ms1_intensities = []
-            peaks_which_isotopes = []
-            peaks_which_adducts = []
-            if mzs is not None:
-                chem_mzs = []
-                chem_intensities = []
-                for items in mzs:
-                    peak_mz, peak_intensity, peak_ms1_int, peak_isotope, \
-                    peak_adduct = items
-                    if (min_measurement_mz <= peak_mz <= max_measurement_mz) \
-                            and peak_intensity > 0:
-                        chem_mzs.append(peak_mz)
-                        chem_intensities.append(peak_intensity)
-                        p = Peak(peak_mz, scan_time, peak_intensity,
-                                 use_ms_level)
-                        peaks.append(p)
-                        peaks_ms1_intensities.append(peak_ms1_int)
-                        peaks_which_isotopes.append(peak_isotope)
-                        peaks_which_adducts.append(peak_adduct)
-                scan_mzs.extend(chem_mzs)
-                scan_intensities.extend(chem_intensities)
-            # for benchmarking purpose
-            if len(peaks) > 0:
-                frag = ScanEvent(chemical, scan_time, use_ms_level, peaks,
-                                 scan_id,
-                                 parents_intensity=peaks_ms1_intensities,
-                                 parent_adduct=peaks_which_adducts,
-                                 parent_isotope=peaks_which_isotopes,
-                                 precursor_mz=params.get(
-                                     ScanParameters.PRECURSOR_MZ),
-                                 isolation_window=isolation_windows,
-                                 scan_params=params)
-                frag_events.append(frag)
+        # generate peaks for all valid chemicals that appear at scan time
+        chems, chemical_peaks = self._get_chem_peaks(isolation_windows, scan_time,
+                                                     use_ms_level)
+
+        # post-processing:
+        # - add noise to generate peak m/z and intensity values
+        # - filter invalid values
+        # - create a fragmentation event for valid peaks in a scan
+        scan_mzs, scan_intensities, frag_events = self._get_scan_post_processing(
+            chemical_peaks, chems, isolation_windows, max_measurement_mz, min_measurement_mz,
+            ms_level, params, scan_id, scan_time, use_ms_level)
         self.fragmentation_events.extend(frag_events)
 
+        # add spike noise to the scan
         # FIXME: this here means spike noise is not a chemical and cannot be fragmented
         # (even though it appears in the scan). Is this the best thing to do?
         if self.spike_noise is not None:
             spike_mzs, spike_intensities = self.spike_noise.sample(
                 min_measurement_mz, max_measurement_mz)
-            scan_mzs.extend(spike_mzs)
-            scan_intensities.extend(spike_intensities)
+            scan_mzs = np.concatenate([scan_mzs, spike_mzs])
+            scan_intensities = np.concatenate([scan_intensities, spike_intensities])
 
+        # for compatibility with old codes
+        frag_events = None if len(frag_events) == 0 else frag_events
+
+        # finally generate a Scan object
         scan_mzs = np.array(scan_mzs)
         scan_intensities = np.array(scan_intensities)
-
-        if len(frag_events) == 0: # for compatibility with old codes
-            frag_events = None
-
         sc = Scan(scan_id, scan_mzs, scan_intensities, ms_level, scan_time,
                   scan_duration=None, scan_params=params,
                   fragevent=frag_events)
@@ -705,104 +645,277 @@ class IndependentMassSpectrometer():
         # we've notified the controller that this scan is produced.
         return sc
 
-    def _get_chem_indices(self, query_rt):
-        rtmin_check = self.chrom_min_rts <= query_rt
-        rtmax_check = query_rt <= self.chrom_max_rts
-        idx = np.nonzero(rtmin_check & rtmax_check)[0]
-        return idx
+    def _get_isolation_windows(self, max_measurement_mz, min_measurement_mz, ms_level, params):
+        # if ms1 then we scan the whole range of m/z
+        if ms_level == 1:
+            isolation_windows = params.get(ScanParameters.ISOLATION_WINDOWS)
+            if isolation_windows is None:
+                isolation_windows = [[(min_measurement_mz, max_measurement_mz)]]
+            assert isolation_windows[0][0][0] == min_measurement_mz
+            assert isolation_windows[0][0][1] == max_measurement_mz
 
-    def _get_all_mz_peaks(self, chemical, query_rt, ms_level,
-                          isolation_windows):
-        # check if the chemical RT matches the current query RT
-        if not self._rt_match(chemical, query_rt):
-            return None
-
-        # if yes, then gather all the peaks from all the isotopes and
-        # adduct of this chemical
-        mz_peaks = []
-        for which_isotope in range(len(chemical.isotopes)):
-            for which_adduct in range(len(self._get_adducts(chemical))):
-                peaks = self._get_mz_peaks(chemical, query_rt, ms_level,
-                                           isolation_windows, which_isotope,
-                                           which_adduct)
-                mz_peaks.extend(peaks)
-
-        # if no peaks generated, then just return None
-        if len(mz_peaks) == 0:
-            return None
-        # apply noise if any
-        noisy_mz_peaks = []
-        for i in range(len(mz_peaks)):
-            original_mz = mz_peaks[i][0]
-            original_intensity = mz_peaks[i][1]
-            noisy_mz = self.mz_noise.get(original_mz, ms_level)
-            noisy_intensity = self.intensity_noise.get(original_intensity,
-                                                       ms_level)
-            noisy_mz_peaks.append((noisy_mz, noisy_intensity, mz_peaks[i][2],
-                                   mz_peaks[i][3], mz_peaks[i][4]))
-        return noisy_mz_peaks
-
-    def _get_mz_peaks(self, chemical, query_rt, ms_level, isolation_windows,
-                      which_isotope, which_adduct):
-        # EXAMPLE OF USE OF DEFINITION: if we wants to do an ms2 scan on a
-        # chemical. we would first have ms_level=2 and the chemicals
-        # ms_level =1. So we would go to the "else". We then check the ms1
-        # window matched. It then would loop through
-        # the children who have ms_level = 2. So we then go to second elif
-        # and return the mz and intensity of each ms2 fragment
-        mz_peaks = []
-        if ms_level == 1 and chemical.ms_level == 1:  # fragment ms1 peaks
-            # returns ms1 peaks if chemical is has ms_level = 1 and
-            # scan is an ms1 scan
-            if not (which_isotope > 0 and which_adduct > 0):
-                # rechecks isolations window if not monoisotopic and
-                # "M + H" adduct
-                if self._isolation_match(chemical, query_rt,
-                                         isolation_windows[0], which_isotope,
-                                         which_adduct):
-                    intensity = self._get_intensity(
-                        chemical, query_rt, which_isotope, which_adduct)
-                    mz = self._get_mz(chemical, query_rt, which_isotope,
-                                      which_adduct)
-                    mz_peaks.extend([(mz, intensity, None, None, None)])
-        elif ms_level == chemical.ms_level:
-            # returns ms2 fragments if chemical and scan are both ms2,
-            # returns ms3 fragments if chemical and scan are both ms3, etc, etc
-            ms1_intensity = self._get_intensity(chemical.parent, query_rt,
-                                                which_isotope, which_adduct)
-            intensity = self._get_intensity(chemical, query_rt, which_isotope,
-                                            which_adduct)
-            mz = self._get_mz(chemical, query_rt, which_isotope, which_adduct)
-            if self.isolation_transition_window == 'gaussian':
-                parent_mz = self._get_mz(chemical.parent, query_rt,
-                                         which_isotope, which_adduct)
-                norm_dist = scipy.stats.norm(
-                    0, self.isolation_transition_window_params[0])
-                scale_factor = norm_dist.pdf(
-                    parent_mz - sum(isolation_windows[ms_level - 2][0]) / 2)
-                scale_factor /= scipy.stats.norm(
-                    0, self.isolation_transition_window_params[0]).pdf(0)
-                intensity *= scale_factor
-            return_values = [(mz, intensity, ms1_intensity, which_isotope,
-                              which_adduct)]
-            return return_values
-            # return extra information here for logging
-            # TODO: Potential improve how the isotope spectra are generated
         else:
-            # check isolation window for ms2+ scans, queries children
-            # if isolation windows ok
-            isolated = self._isolation_match(
-                chemical, query_rt, isolation_windows[chemical.ms_level - 1],
-                which_isotope, which_adduct)
-            if isolated and chemical.children is not None:
-                for i in range(len(chemical.children)):
-                    mz_peaks.extend(
-                        self._get_mz_peaks(chemical.children[i], query_rt,
-                                           ms_level, isolation_windows,
-                                           which_isotope, which_adduct))
-            else:
-                return []
-        return mz_peaks
+            # if ms2 then we check if the isolation window parameter is specified
+            # if not then we compute from the precursor mz and isolation width
+            isolation_windows = params.compute_isolation_windows()
+
+        # make isolation windows to be a numpy array for convenience later
+        if not isinstance(isolation_windows, np.ndarray):
+            isolation_windows = np.array(isolation_windows)
+        return isolation_windows
+
+    def _get_chem_peaks(self, isolation_windows, scan_time, use_ms_level):
+        assert use_ms_level in [1, 2]
+        chems = self.chemicals.next_chems(scan_time)
+        if use_ms_level == 1:
+            chemical_peaks = self._get_chem_peaks_for_ms1(chems, self.chem_data_collector,
+                                                          scan_time)
+        elif use_ms_level == 2:
+            chemical_peaks = self._get_chem_peaks_for_ms2(chems, self.chem_data_collector,
+                                                          isolation_windows, scan_time)
+        # assert len(chems) == len(chemical_peaks)
+        return chems, chemical_peaks
+
+    def _get_chem_peaks_for_ms1(self, chems, cdc, scan_time):
+        all_chems, which_isotopes, which_adducts, peaks = generate_chem_ms1_peaks_for_ms1(
+            chems, scan_time, cdc)
+
+        # convert to defaultdict, key: chem, value: list of peaks for that chem
+        chemical_peaks = defaultdict(list)
+        for chemical, peak in zip(all_chems, peaks):
+            chemical_peaks[chemical].append(peak)
+        return chemical_peaks
+
+    def _get_chem_peaks_for_ms2(self, chems, cdc, isolation_windows, scan_time):
+
+        isolated_chems, isolated_which_adducts, isolated_which_isotopes = \
+            self._isolate_chems_for_fragmentation(
+                chems, cdc, isolation_windows, scan_time)
+
+        chemical_peaks = self._get_children_spectra(chems, isolated_chems,
+                                                    isolated_which_adducts,
+                                                    isolated_which_isotopes, isolation_windows,
+                                                    scan_time)
+        return chemical_peaks
+
+    def _isolate_chems_for_fragmentation(self, chems, cdc, isolation_windows,
+                                         scan_time):
+
+        _, which_isotopes, which_adducts, peaks = generate_chem_ms1_peaks_for_ms2(
+            chems, scan_time, cdc)
+
+        if len(peaks) > 0:
+            mzs = peaks[:, 0]
+        else:
+            mzs = np.array([])
+
+        # FIXME: only support one window, maybe not correct
+        # original code is
+        #             isolated = isolation_match(mz, isolation_windows[chemical.ms_level - 1])
+        assert len(isolation_windows[0]) == 1, 'Multiple isolation windows not supported'
+        lower_bound, upper_bound = isolation_windows[0][0]
+
+        isolated = np.logical_and(mzs >= lower_bound, mzs <= upper_bound)
+        isolated_chems = chems[isolated]
+        isolated_which_adducts = which_adducts[isolated]
+        isolated_which_isotopes = which_isotopes[isolated]
+        return isolated_chems, isolated_which_adducts, isolated_which_isotopes
+
+    def _get_children_spectra(self, chems, isolated_chems, isolated_which_adducts,
+                              isolated_which_isotopes, isolation_windows, scan_time):
+
+        chemical_peaks = defaultdict(list)
+
+        # TODO: vectorise this
+        query_rt = scan_time
+        for chemical, which_adduct, which_isotope in zip(
+                isolated_chems, isolated_which_adducts, isolated_which_isotopes):
+
+            ms1_intensity = self._get_intensity_ms1(chemical, query_rt,
+                                                    which_isotope, which_adduct)
+
+            mz_peaks = []
+            for i in range(len(chemical.children)):
+                mz_peaks.extend(
+                    self._get_mz_peaks_child(chemical.children[i], ms1_intensity,
+                                             which_isotope, which_adduct))
+
+            for row in mz_peaks:
+                chemical_peaks[chemical].append(row)
+
+        return chemical_peaks
+
+    def _get_scan_post_processing(self, chemical_peaks, chems, isolation_windows,
+                                  max_measurement_mz, min_measurement_mz, ms_level, params,
+                                  scan_id, scan_time, use_ms_level):
+
+        scan_mzs = np.array([])  # all the mzs values in this scan
+        scan_intensities = np.array([])  # all the intensity values in this scan
+
+        all_peaks = []
+        frag_events = []
+        for chemical in chems:
+            peaks = chemical_peaks[chemical]
+            if len(peaks) == 0:
+                continue
+
+            # apply noise if any
+            peaks = np.array(peaks)
+            peaks = self.add_noise_and_filter(max_measurement_mz, min_measurement_mz,
+                                              ms_level, peaks)
+
+            if len(peaks) > 0:  # if after filter, some peaks are left
+                all_peaks.append(peaks)
+
+                # for synthetic performance evaluation
+                frag = self._get_frag_event(chemical, isolation_windows, peaks, params,
+                                            scan_id, scan_time, use_ms_level)
+                frag_events.append(frag)
+
+        # combine generated peaks, if any
+        if len(all_peaks) > 0:
+            concat_peaks = np.concatenate(all_peaks, axis=0)
+            scan_mzs = concat_peaks[:, PEAKS_MZ_IDX]
+            scan_intensities = concat_peaks[:, PEAKS_INTENSITY_IDX]
+        return scan_mzs, scan_intensities, frag_events
+
+    def add_noise_and_filter(self, max_measurement_mz, min_measurement_mz, ms_level, peaks):
+
+        # Non-vectorised, but faster
+        noise_peaks = []
+        for i in range(len(peaks)):
+            original_mz = peaks[i][PEAKS_MZ_IDX]
+            original_intensity = peaks[i][PEAKS_INTENSITY_IDX]
+            noisy_mz = self.mz_noise.get(original_mz, ms_level)
+            noisy_intensity = self.intensity_noise.get(original_intensity, ms_level)
+
+            if (min_measurement_mz <= noisy_mz <= max_measurement_mz) and (noisy_intensity > 0):
+                noise_peaks.append(
+                    (noisy_mz, noisy_intensity,
+                     peaks[i][PEAKS_MS1_INTENSITY_IDX],
+                     peaks[i][PEAKS_WHICH_ISOTOPE_IDX],
+                     peaks[i][PEAKS_WHICH_ADDUCT_IDX])
+                )
+        noise_peaks = np.array(noise_peaks)
+
+        # Vectorised but SLOW?!
+        # noisy_mz = np.zeros(peaks.shape[0])
+        # noisy_intensity = np.zeros(peaks.shape[0])
+        # for i in range(peaks.shape[0]):
+        #     original_mz = peaks[i, PEAKS_MZ_IDX]
+        #     original_intensity = peaks[i, PEAKS_INTENSITY_IDX]
+        #     noisy_mz[i] = self.mz_noise.get(original_mz, ms_level)
+        #     noisy_intensity[i] = self.intensity_noise.get(original_intensity, ms_level)
+        #
+        # peaks[:, PEAKS_MZ_IDX] = noisy_mz
+        # peaks[:, PEAKS_INTENSITY_IDX] = noisy_intensity
+        # mask = (min_measurement_mz <= peaks[:, PEAKS_MZ_IDX]) & (
+        #             peaks[:, PEAKS_MZ_IDX] <= max_measurement_mz) & (
+        #                    peaks[:, PEAKS_INTENSITY_IDX] > 0)
+        # noise_peaks = peaks[mask, :]
+
+        return noise_peaks
+
+    def _get_frag_event(self, chemical, isolation_windows, peaks, params, scan_id, scan_time,
+                        use_ms_level):
+
+        # Slow. Not sure if it's ever needed.
+        # scan_peaks = [ScanEventPeak(peak[PEAKS_MZ_IDX],
+        #                             scan_time,
+        #                             peak[PEAKS_INTENSITY_IDX],
+        #                             use_ms_level) for peak in peaks]
+        scan_peaks = None
+        parents_intensity = None
+        parent_adduct = None
+        parent_isotope = None
+        precursor_mz = None
+        if use_ms_level == 2:
+            parents_intensity = peaks[:, PEAKS_MS1_INTENSITY_IDX]
+            parent_adduct = peaks[:, PEAKS_WHICH_ISOTOPE_IDX]
+            parent_isotope = peaks[:, PEAKS_WHICH_ADDUCT_IDX]
+            precursor_mz = params.get(ScanParameters.PRECURSOR_MZ)
+
+        frag = ScanEvent(chemical, scan_time, use_ms_level, scan_peaks,
+                         scan_id,
+                         parents_intensity=parents_intensity,
+                         parent_adduct=parent_adduct,
+                         parent_isotope=parent_isotope,
+                         precursor_mz=precursor_mz,
+                         isolation_window=isolation_windows,
+                         scan_params=params)
+        return frag
+
+    def _get_mz_peaks_child(self, chemical, ms1_intensity,
+                            which_isotope, which_adduct):
+
+        # generate MS2 scan from a child MS2 chemical
+        assert chemical.ms_level == 2
+
+        if self.skip_ms2_spectra_generation:
+            # small speed-up by not actually generating the proper ms2 spectra here
+            ms2_intensity = 1000
+            mz = 100
+            return_values = [(mz, ms2_intensity, ms1_intensity, which_isotope, which_adduct)]
+            return return_values
+
+        # returns ms2 fragments if chemical and scan are both ms2,
+        # returns ms3 fragments if chemical and scan are both ms3, etc, etc
+        ms2_intensity = self._get_intensity_ms2(chemical, ms1_intensity)
+        mz = self.get_ms2_peaks_from_chemical(chemical, which_isotope, which_adduct)
+
+        # experimental gaussian isolation window function, maybe can be removed
+        # if self.isolation_transition_window == 'gaussian':
+        #     intensity = self.gaussian_isolation(chemical, query_rt, which_isotope,
+        #                                         which_adduct, isolation_windows, ms_level,
+        #                                         intensity)
+
+        return_values = [(mz, ms2_intensity, ms1_intensity, which_isotope, which_adduct)]
+        return return_values
+
+    def _get_intensity_ms1(self, chemical, query_rt, which_isotope, which_adduct):
+        assert chemical.ms_level == 1
+        intensity = chemical.isotopes[which_isotope][1] * \
+                    self._get_adducts(chemical)[which_adduct][1] * \
+                    chemical.max_intensity
+        return intensity * chemical.chromatogram.get_relative_intensity(
+            query_rt - chemical.rt)
+
+    def _get_intensity_ms2(self, chemical, ms1_intensity):
+        assert chemical.ms_level == 2
+        prop = chemical.parent_mass_prop
+        if isinstance(prop, np.ndarray):
+            prop = prop[0]
+        intensity = ms1_intensity * prop * chemical.prop_ms2_mass
+        return intensity
+
+    def get_ms2_peaks_from_chemical(self, chemical, which_isotope, which_adduct):
+        ms1_parent = chemical
+        while ms1_parent.ms_level != 1:
+            ms1_parent = chemical.parent
+        ms1_parent_isotopes = ms1_parent.isotopes
+
+        # TODO: Potential improve how the isotope spectra are generated
+        mz = chemical.isotopes[0][0]
+        parent_adduct = chemical.parent.adducts[self.ionisation_mode]
+        adduct = parent_adduct[which_adduct][0]
+        mul, add = ADDUCT_TERMS[adduct]
+
+        mz_value = get_mz_msn(mz, mul, add, ms1_parent_isotopes, which_isotope)
+        return mz_value
+
+    # experimental code, maybe can remove
+    # def gaussian_isolation(self, chemical, query_rt, which_isotope, which_adduct,
+    #                        isolation_windows, ms_level, intensity):
+    #     parent_mz = self.get_chemical_mz_ms1(chemical.parent, query_rt,
+    #                                          which_isotope, which_adduct)
+    #     norm_dist = scipy.stats.norm(
+    #         0, self.isolation_transition_window_params[0])
+    #     scale_factor = norm_dist.pdf(
+    #         parent_mz - sum(isolation_windows[ms_level - 2][0]) / 2)
+    #     scale_factor /= scipy.stats.norm(
+    #         0, self.isolation_transition_window_params[0]).pdf(0)
+    #     intensity *= scale_factor
+    #     return intensity
 
     def _get_adducts(self, chemical):
         if chemical.ms_level == 1:
@@ -813,51 +926,22 @@ class IndependentMassSpectrometer():
         else:
             return self._get_adducts(chemical.parent)
 
-    def _rt_match(self, chemical, query_rt):
-        return chemical.ms_level != 1 or chemical.chromatogram._rt_match(
-            query_rt - chemical.rt)
+    def get_chemical_mz_ms1(self, chemical, query_rt, which_isotope, which_adduct):
+        chrom = chemical.chromatogram
+        chrom_type = chrom.get_chrom_type()
 
-    def _get_intensity(self, chemical, query_rt, which_isotope, which_adduct):
-        if chemical.ms_level == 1:
-            intensity = chemical.isotopes[which_isotope][1] * \
-                        self._get_adducts(chemical)[which_adduct][1] * \
-                        chemical.max_intensity
-            return intensity * chemical.chromatogram.get_relative_intensity(
-                query_rt - chemical.rt)
-        else:
-            prop = chemical.parent_mass_prop
-            if isinstance(prop, np.ndarray):
-                prop = prop[0]
-            intensity = self._get_intensity(
-                chemical.parent, query_rt, which_isotope, which_adduct)
-            intensity = intensity * prop * chemical.prop_ms2_mass
-            return intensity
+        mz = chemical.isotopes[which_isotope][0]
+        adduct = chemical.adducts[self.ionisation_mode][which_adduct][0]
+        mul, add = ADDUCT_TERMS[adduct]
 
-    def _get_mz(self, chemical, query_rt, which_isotope, which_adduct):
-        if chemical.ms_level == 1:
-            return (adduct_transformation(chemical.isotopes[which_isotope][0],
-                                          self._get_adducts(chemical)[
-                                              which_adduct][0]) +
-                    chemical.chromatogram.get_relative_mz(
-                        query_rt - chemical.rt))
-        else:
-            ms1_parent = chemical
-            while ms1_parent.ms_level != 1:
-                ms1_parent = chemical.parent
-            isotope_transformation = ms1_parent.isotopes[which_isotope][0] - \
-                                     ms1_parent.isotopes[0][0]
-            # TODO: Needs improving
-            return (adduct_transformation(chemical.isotopes[0][0],
-                                          self._get_adducts(chemical)[
-                                              which_adduct][
-                                              0]) + isotope_transformation)
+        try:
+            rts = chrom.rts
+            mzs = chrom.mzs
+        except AttributeError:
+            rts = np.array([])
+            mzs = np.array([])
 
-    def _isolation_match(self, chemical, query_rt, isolation_windows,
-                         which_isotope, which_adduct):
-        # assumes list is formated like:
-        # [(min_1,max_1),(min_2,max_2),...]
-        for window in isolation_windows:
-            if window[0] < self._get_mz(chemical, query_rt, which_isotope,
-                                        which_adduct) <= window[1]:
-                return True
-        return False
+        mz_value = get_mz_ms1(mz, mul, add, chrom_type, query_rt, chemical.rt,
+                              chrom.min_rt, chrom.max_rt,
+                              rts, mzs)
+        return mz_value
