@@ -21,6 +21,7 @@ from mass_spec_utils.data_import.mzmine import (
 )
 from mass_spec_utils.data_import.mzml import MZMLFile
 from mass_spec_utils.data_processing.alignment import Peak, PeakSet
+from numba import njit
 from scipy.stats import pearsonr
 
 from vimms.Box import GenericBox
@@ -33,7 +34,6 @@ from vimms.Evaluation import (
     load_peakonly_boxes, load_xcms_boxes, 
     get_precursor_intensities
 )
-from vimms.MassSpec import Scan
 
 
 class Roi():
@@ -63,43 +63,41 @@ class Roi():
         self.fragmentation_events = []
         self.fragmentation_intensities = []
         self.max_fragmentation_intensity = 0.0
-        self.mz_list = self._to_list(mz)
-        self.rt_list = self._to_list(rt)
-        self.intensity_list = self._to_list(intensity)
+        self.mz_list = [mz]
+        self.rt_list = [rt]
+        self.intensity_list = [intensity]
         self.n = len(self.mz_list)
         self.mz_sum = sum(self.mz_list)
-        self.length_in_seconds = self.rt_list[-1] - self.rt_list[0]
+        self.mean_mz = self.calculate_mean_mz()
+        self.min_rt = self.rt_list[0]
+        self.max_rt = self.rt_list[-1]
+        self.length_in_seconds = self.max_rt - self.min_rt
         self.is_fragmented = False
         self.can_fragment = True
-        
-        self.min_rt, self.max_rt = min(self.rt_list), max(self.rt_list)
-        self.min_mz, self.max_mz = min(self.mz_list), max(self.mz_list)
-        self.min_intensity, self.max_intensity = min(self.intensity_list), max(self.intensity_list)
+        self.last_frag_rt = None
 
-    def fragmented(self):
+    def fragmented(self, rt):
         """
         Sets flags to indicate that this ROI can or has been fragmented
         """
         self.is_fragmented = True
         self.can_fragment = True
+        self.last_frag_rt = rt
 
-    def get_mean_mz(self):
-        """
-        Returns the mean m/z values of points in this ROI
-        """
+    def calculate_mean_mz(self):
         return self.mz_sum / self.n
 
     def get_max_intensity(self):
         """
         Returns the maximum intensity value of this ROI
         """
-        return self.max_intensity
+        return max(self.intensity_list)
 
     def get_min_intensity(self):
         """
         Returns the minimum intensity value of this ROI
         """
-        return self.min_intensity
+        return min(self.intensity_list)
 
     def get_autocorrelation(self, lag=1):
         """
@@ -130,10 +128,10 @@ class Roi():
         self.intensity_list.append(intensity)
         self.mz_sum += mz
         self.n += 1
-        self.length_in_seconds = self.rt_list[-1] - self.rt_list[0]
-        self.min_rt, self.max_rt = min(self.min_rt, rt), max(self.max_rt, rt)
-        self.min_mz, self.max_mz = min(self.min_mz, mz), max(self.max_mz, mz)
-        self.min_intensity, self.max_intensity = min(self.min_intensity, intensity), max(self.max_intensity, intensity)
+        self.mean_mz = self.calculate_mean_mz()
+        self.min_rt = self.rt_list[0]
+        self.max_rt = self.rt_list[-1]
+        self.length_in_seconds = self.max_rt - self.min_rt
 
     def add_fragmentation_event(self, scan, precursor_intensity):
         """
@@ -161,24 +159,57 @@ class Roi():
                                       np.array(self.intensity_list))
         return chrom
 
-    def to_box(self, min_rt_width, min_mz_width, rt_shift=0, mz_shift=0):
+    @staticmethod
+    def _set_fixed_bounds_for_box(dist, minm, maxm):
+        if(not dist is None):
+            mid = (minm + maxm) / 2
+            new_minm = mid - dist
+            new_maxm = mid + dist
+        else:
+            new_minm, new_maxm = minm, maxm
+        return new_minm, new_maxm
+
+    def to_box(self, 
+               min_rt_width, 
+               min_mz_width,
+               fixed_rt_dist=None,
+               fixed_mz_dist=None,
+               rt_shift=0, 
+               mz_shift=0):
         """
         Returns a generic box representation of this ROI
 
         Args:
             min_rt_width: minimum RT width of the box
             min_mz_width: minimum m/z width of the box
+            fixed_rt_dist: if not set to None, overrides the default rt-width (in seconds)
+            of the box, setting it to twice the parameter value. If set to None uses the 
+            maximum rt-range of points belonging to this RoI as the box's rt-width
+            fixed_mz_width: if not set to None, overrides the default mz-width (in ppm) of 
+            the box, setting it twice the parameter value. If set to None uses the 
+            maximum mz-range of points belonging to this RoI as the box's rt-width
             rt_shift: shift in retention time, if any
             mz_shift: shift in m/z, if any
 
         Returns: a [vimms.Box.GenericBox][] object.
 
         """
+        min_rt, max_rt = min(self.rt_list), max(self.rt_list)
+        min_mz, max_mz = min(self.mz_list), max(self.mz_list)
+
+        min_rt, max_rt = self._set_fixed_bounds_for_box(fixed_rt_dist, min_rt, max_rt)
+        dalton_dist = (
+            self.mean_mz * fixed_mz_dist / 1E6
+            if not fixed_mz_dist is None
+            else None
+        )
+        min_mz, max_mz = self._set_fixed_bounds_for_box(dalton_dist, min_mz, max_mz)
+        
         return GenericBox(
-                    self.min_rt + rt_shift, 
-                    self.max_rt + rt_shift, 
-                    self.min_mz + mz_shift,
-                    self.max_mz + mz_shift,
+                    min_rt + rt_shift, 
+                    max_rt + rt_shift, 
+                    min_mz + mz_shift,
+                    max_mz + mz_shift,
                     min_xwidth=min_rt_width, 
                     min_ywidth=min_mz_width, 
                     intensity=self.max_fragmentation_intensity, 
@@ -231,19 +262,6 @@ class Roi():
         """
         return self.mz_list[-1], self.rt_list[-1], self.intensity_list[-1]
 
-    def _to_list(self, val):
-        """
-        Ensures that the value passed in is a list
-
-        Args:
-            val: the value to check, can be either a list or a single value
-
-        Returns: val, but always as a list
-
-        """
-        values = val if type(val) == list else [val]
-        return values
-
     def __getitem__(self, idx):
         """
         Returns a single point in this ROI at the
@@ -268,7 +286,7 @@ class Roi():
         Returns: comparison is done by mean m/z of ROIs
 
         """
-        return self.get_mean_mz() <= other.get_mean_mz()
+        return self.mean_mz <= other.mean_mz
 
     def __repr__(self):
         """
@@ -312,25 +330,29 @@ class SmartRoi(Roi):
             id: the ID of this ROI
         """
         super().__init__(mz, rt, intensity, id=id)
+        self.max_at_frag = 0.0
+        self.max_since_last_frag = 0.0
         self.params = smartroi_params
 
         if self.params.initial_length_seconds > 0:
             self.status = SmartRoi.INITIAL_WAITING
-            self.set_can_fragment(False)
+            self.can_fragment = False
         else:
             self.status = SmartRoi.CAN_FRAGMENT
-            self.set_can_fragment(True)
+            self.can_fragment = True
 
         self.min_frag_intensity = None
-        self.intensity_diff = 0
 
-    def fragmented(self):
+    def fragmented(self, rt):
         """
         Sets this SmartROI as having been fragmented
         """
         self.is_fragmented = True
-        self.set_can_fragment(False)
+        self.can_fragment = False
+        self.last_frag_rt = rt
         self.fragmented_index = len(self.mz_list) - 1
+        self.max_at_frag = self.intensity_list[self.fragmented_index]
+        self.max_since_last_frag = self.max_at_frag
         self.status = SmartRoi.AFTER_FRAGMENT
 
     def get_status(self):
@@ -360,42 +382,49 @@ class SmartRoi(Roi):
 
         """
         super().add(mz, rt, intensity)
+        if intensity > self.max_since_last_frag:
+            self.max_since_last_frag = intensity
+
         if self.status == SmartRoi.INITIAL_WAITING:
             if self.length_in_seconds >= self.params.initial_length_seconds:
                 self.status = SmartRoi.CAN_FRAGMENT
-                self.set_can_fragment(True)
+                self.can_fragment = True
+
         elif self.status == SmartRoi.AFTER_FRAGMENT:
-            # in a period after a fragmentation has happened
-            # if enough time has elapsed, reset everything
-            if self.rt_list[-1] - self.rt_list[
-                self.fragmented_index] > self.params.reset_length_seconds:
-                self.status = SmartRoi.CAN_FRAGMENT
-                self.set_can_fragment(True)
-            elif self.rt_list[-1] - self.rt_list[
-                self.fragmented_index] > self.params.dew:
-                # standard DEW has expired
-                # find the min intensity since the frag
-                # check current intensity -- if it is 5* when we fragmented,
-                # we can go again
-                min_since_frag = min(
-                    self.intensity_list[self.fragmented_index:])
-                if self.intensity_list[
-                    -1] > min_since_frag * self.params.intensity_increase_factor:
-                    self.status = SmartRoi.CAN_FRAGMENT
-                    self.set_can_fragment(True)
-                elif self.intensity_list[-1] < self.params.drop_perc * \
-                        self.intensity_list[self.fragmented_index]:
-                    # signal has dropped, but ROI still exists.
-                    self.status = SmartRoi.CAN_FRAGMENT
-                    self.set_can_fragment(True)
+            self.set_smartroi_rules()
 
         # code below never happens
-        elif self.status == SmartRoi.POST_PEAK:
-            if self.rt_list[-1] - self.rt_list[
-                self.fragmented_index] > self.params.dew:
-                if self.intensity_list[-1] > self.min_frag_intensity:
-                    self.status = SmartRoi.CAN_FRAGMENT
-                    self.set_can_fragment(True)
+        # elif self.status == SmartRoi.POST_PEAK:
+        #     if self.rt_list[-1] - self.rt_list[
+        #         self.fragmented_index] > self.params.dew:
+        #         if self.intensity_list[-1] > self.min_frag_intensity:
+        #             self.status = SmartRoi.CAN_FRAGMENT
+        #             self.set_can_fragment(True)
+
+    def set_smartroi_rules(self):
+        # in a period after a fragmentation has happened
+        # if enough time has elapsed, reset everything
+        if self.rt_list[-1] - self.rt_list[self.fragmented_index] > \
+                self.params.reset_length_seconds:
+            self.status = SmartRoi.CAN_FRAGMENT
+            self.can_fragment = True
+
+        elif self.rt_list[-1] - self.rt_list[self.fragmented_index] > self.params.dew:
+            # standard DEW has expired so apply smartroi rules
+            frag = self.intensity_list[self.fragmented_index]
+
+            # too slow to recompute this each time
+            # max_since_frag = max(self.intensity_list[self.fragmented_index:])
+            # assert self.max_since_last_frag == max_since_frag
+
+            if self.intensity_list[-1] > self.params.intensity_increase_factor * frag:
+                self.status = SmartRoi.CAN_FRAGMENT
+                self.can_fragment = True
+
+            elif self.intensity_list[-1] < self.params.drop_perc * self.max_since_last_frag:
+                # signal has dropped, but ROI still exists.
+                self.status = SmartRoi.CAN_FRAGMENT
+                self.can_fragment = True
 
     def get_can_fragment(self):
         """
@@ -415,12 +444,6 @@ class SmartRoi(Roi):
 
         """
         self.can_fragment = status
-        try:
-            self.intensity_diff = abs(
-                self.intensity_list[-1] - self.intensity_list[
-                    self.fragmented_index])
-        except AttributeError:  # no fragmented index
-            self.intensity_diff = 0
 
 
 class SmartRoiParams():
@@ -522,10 +545,6 @@ class RoiBuilder():
         self.dead_roi = []
         self.junk_roi = []
 
-        # FIXME: not sure if we actually need the two properties below?
-        self.live_roi_fragmented = []
-        self.live_roi_last_rt = []  # last fragmentation time of ROI
-
         # fragmentation to Roi dictionaries
         self.frag_roi_dicts = []  # scan_id, roi_id, precursor_intensity
         self.roi_id_counter = 0
@@ -549,12 +568,7 @@ class RoiBuilder():
             # Sort ROIs in live_roi according to their m/z values.
             # Ensure that the live roi fragmented flags and the last RT
             # are also consistent with the sorting order.
-            order = np.argsort(self.live_roi)
             self.live_roi.sort()
-            self.live_roi_fragmented = np.array(self.live_roi_fragmented)[
-                order].tolist()
-            self.live_roi_last_rt = np.array(self.live_roi_last_rt)[
-                order].tolist()
 
             # Current scan retention time of the MS1 scan is the RT of all
             # points in this scan
@@ -609,13 +623,6 @@ class RoiBuilder():
                         self.roi_id_counter += 1
                         bisect.insort_right(self.live_roi, new_roi)
 
-                        # Set the fragmented flag of a new ROI to False.
-                        # Also set its last fragmented time to None
-                        self.live_roi_fragmented.insert(
-                            self.live_roi.index(new_roi), False)
-                        self.live_roi_last_rt.insert(
-                            self.live_roi.index(new_roi), None)
-
             # Separate the ROIs that have not been grown into dead or junk ROIs
             # Dead ROIs are longer than self.min_roi_length but they haven't
             # been grown. Junk ROIs are too short and not grown.
@@ -634,8 +641,6 @@ class RoiBuilder():
                     # Remove not-grown ROI from the list of live ROIs
                     pos = self.live_roi.index(roi)
                     del self.live_roi[pos]
-                    del self.live_roi_fragmented[pos]
-                    del self.live_roi_last_rt[pos]
 
                     # this ROI is either dead or junk, so delete from skip count
                     # as we don't need to track it anymore
@@ -659,7 +664,7 @@ class RoiBuilder():
                       mz ROI into
             mz_tol: m/z tolerance. This is the window above and below the
                     mean_mz of the RoI.
-                    E.g. if mz_tol = 1 Da, then it looks plus and minus 1Da
+                    E.g. if mz_tol = 10 ppm, then it looks plus and minus 10 ppm
 
         Returns: the ROI that has been matched, or None if not found.
 
@@ -669,23 +674,32 @@ class RoiBuilder():
         pos = bisect.bisect_right(roi_list, mz)
 
         if pos == len(roi_list):
-            dist_left = 1e6 * (mz.get_mean_mz() - roi_list[
-                pos - 1].get_mean_mz()) / mz.get_mean_mz()
+            dist_left = 1e6 * (
+                mz.mean_mz - roi_list[pos - 1].mean_mz
+            ) / mz.mean_mz
+            
             if dist_left < mz_tol:
                 return roi_list[pos - 1]
             else:
                 return None
+        
         elif pos == 0:
-            dist_right = 1e6 * (roi_list[pos].get_mean_mz() - mz.get_mean_mz()) / mz.get_mean_mz()
+            dist_right = 1e6 * (
+                roi_list[pos].mean_mz - mz.mean_mz
+            ) / mz.mean_mz
+            
             if dist_right < mz_tol:
                 return roi_list[pos]
             else:
                 return None
+                
         else:
-            dist_left = 1e6 * (mz.get_mean_mz() - roi_list[
-                pos - 1].get_mean_mz()) / mz.get_mean_mz()
-            dist_right = 1e6 * (roi_list[
-                                    pos].get_mean_mz() - mz.get_mean_mz()) / mz.get_mean_mz()
+            dist_left = 1e6 * (
+                mz.mean_mz - roi_list[pos - 1].mean_mz
+            ) / mz.mean_mz
+            dist_right = 1e6 * (
+                roi_list[pos].mean_mz - mz.mean_mz
+            ) / mz.mean_mz
 
             if dist_left < mz_tol < dist_right:
                 return roi_list[pos - 1]
@@ -750,10 +764,7 @@ class RoiBuilder():
         Returns: None
 
         """
-        # updated fragmented list and times
-        self.live_roi_fragmented[i] = True
-        self.live_roi_last_rt[i] = rt
-        self.live_roi[i].fragmented()
+        self.live_roi[i].fragmented(rt)
 
         # Add information on which scan has fragmented this ROI
         self.frag_roi_dicts.append(
@@ -802,13 +813,12 @@ class RoiBuilder():
 
         # intensity check:
         # Keep only the ROIs that can be fragmented above
-        # min_roi_intensity_for_fragmentation threshold.
+        # at_least_one_point_above threshold.
         all_roi = filtered_roi + self.dead_roi
         if self.roi_params.at_least_one_point_above > 0:
             keep = []
             for roi in all_roi:
-                if np.count_nonzero(np.array(
-                        roi.intensity_list) > self.roi_params.at_least_one_point_above) > 0:
+                if any(it > self.roi_params.at_least_one_point_above for it in roi.intensity_list):
                     keep.append(roi)
         else:
             keep = all_roi
@@ -881,7 +891,7 @@ class RoiAligner():
         these_peaks, frag_intensities, temp_boxes = [], [], []
         for i, roi in enumerate(rois):
             source_id = f"{sample_name}_{i}"
-            peak_mz = roi.get_mean_mz()
+            peak_mz = roi.mean_mz
             peak_rt = roi.estimate_apex()
             peak_intensity = roi.get_max_intensity()
             these_peaks.append(
@@ -1020,7 +1030,7 @@ class RoiAligner():
         """
         return np.array(
             [
-                [peakset.get_intensity(filename) for fname in self.files_loaded] 
+                [ps.get_intensity(fname) for fname in self.files_loaded]
                 for ps in self.peaksets
             ], 
             dtype=np.double
@@ -1141,6 +1151,8 @@ def make_roi(input_file, roi_params):
              certain criteria.
 
     """
+    
+    from vimms.MassSpec import Scan #import in fn. body to avoid circular import
 
     run = pymzml.run.Reader(input_file, MS1_Precision=5e-6,
                             extraAccessions=[
@@ -1164,7 +1176,7 @@ def make_roi(input_file, roi_params):
 
             # get the raw peak data from spectrum
             mzs, intensities = spectrum_to_arrays(spectrum)
-
+            
             # update the ROI construction based on the new scan
             scan = Scan(scan_id, mzs, intensities, ms_level,
                         current_ms1_scan_rt)
@@ -1346,7 +1358,7 @@ def update_picked_boxes(picked_boxes, rt_shifts, mz_shifts):
         return picked_boxes
         
     new_boxes = copy.deepcopy(picked_boxes)
-    for box, sec_shift, mz_shift in zip(new_boxes, rt_shifts, min_shifts):
+    for box, sec_shift, mz_shift in zip(new_boxes, rt_shifts, mz_shifts):
         if rt_shifts is not None:
             sec_shift = float(sec_shift)
             min_shift = sec_shift / 60.0
