@@ -2,10 +2,16 @@
 # simple script that loads an msp and an mzml and sees how many of the
 # spectra in the MSP file can be matched to a spectrum in an ms2 scan
 # in the .mzml
+import sys
+
+from vimms.Box import GenericBox
+
+sys.path.append('..')
+sys.path.append('../..')  # if running in this folder
+
 import argparse
 import glob
 import os
-import sys
 
 import numpy as np
 import pandas as pd
@@ -14,10 +20,7 @@ from mass_spec_utils.data_import.mzml import MZMLFile
 from mass_spec_utils.library_matching.spec_libraries import SpectralLibrary
 from mass_spec_utils.library_matching.spectrum import Spectrum, SpectralRecord
 
-from vimms.Common import load_obj
-
-sys.path.append('..')
-sys.path.append('../..')  # if running in this folder
+from vimms.Common import load_obj, ScanParameters, PROTON_MASS
 
 
 def process_block(block, file_name):
@@ -87,41 +90,109 @@ def library_from_msp(msp_file_name):
     return sl
 
 
-def make_queries_from_aligned_msdial(msdial_file_name):
+def make_queries_from_aligned_msdial(msdial_file_name, frag_file=True,
+                                     sample_name=None):
     query_spectra = []
     msdial_df = pd.read_csv(msdial_file_name, sep='\t',
                             index_col='Alignment ID', header=4)
-    for i in range(msdial_df.shape[0]):
-        precursor_mz = msdial_df['Average Mz'][i]
-        peaks = []
-        if msdial_df['MS/MS spectrum'][i] == msdial_df['MS/MS spectrum'][
-                i]:  # checking if nan
-            for info in msdial_df['MS/MS spectrum'][i].split():
-                mz, intensity = info.split(':')
-                peak = np.array([float(mz), float(intensity)])
-                peaks.append(peak)
-            new_spectrum = Spectrum(precursor_mz, peaks)
-            query_spectra.append(new_spectrum)
+    if frag_file:
+        msdial_df = msdial_df[msdial_df['MS/MS assigned'] == True]
+
+    for i, row in msdial_df.iterrows():
+        new_spectrum = aligned_row_to_spectral_record(row, sample_name=sample_name)
+        query_spectra.append(new_spectrum)
     return query_spectra
 
 
+def aligned_row_to_spectral_record(row, sample_name=None):
+    peaks = extract_msdial_spectrum(row, 'MS/MS spectrum')
+    precursor_mz = row['Average Mz']
+    metadata = {
+        'rt_in_minutes': row['Average Rt(min)'],
+        'rt': row['Average Rt(min)'] * 60.0,
+        'names': [row['Metabolite name']],
+        'msms_assigned': row['MS/MS assigned'],
+        'rt_matched': row['RT matched'],
+        'mz_matched': row['m/z matched'],
+        'msms_matched': row['MS/MS matched'],
+        'total_score': row['Total score'],
+        'dot_product': row['Dot product'],
+        'reverse_dot_product': row['Reverse dot product'],
+        'fragment_presence': row['Fragment presence %'],
+        'msms_spectrum': row['MS/MS spectrum'],
+    }
+    if sample_name is not None:
+        sample_cols = row.filter(regex=sample_name)
+        metadata.update({
+            'samples': sample_cols.to_dict(),
+            'sample_count': np.count_nonzero(sample_cols.values)
+        })
+
+    original_file = row['Spectrum reference file name']
+    spectrum_id = 'peak_%.6f' % precursor_mz
+    new_spectrum = SpectralRecord(precursor_mz, peaks, metadata, original_file, spectrum_id)
+    return new_spectrum
+
+
+def extract_msdial_spectrum(row, key):
+    peaks = []
+    try:
+        for info in row[key].split():
+            mz, intensity = info.split(':')
+            peak = np.array([float(mz), float(intensity)])
+            peaks.append(peak)
+    except AttributeError:  # no MS2 spectrum
+        pass
+    return peaks
+
+
 def make_queries_from_msdial(msdial_file_name):
+    original_file = os.path.splitext(msdial_file_name)[0]
     query_spectra = []
     msdial_df = pd.read_csv(msdial_file_name, sep='\t', index_col='PeakID')  #
-    for i in range(msdial_df.shape[0]):
-        precursor_mz = msdial_df['Precursor m/z'][i]
-        peaks = []
-        if msdial_df['MSMS spectrum'][i] == msdial_df['MSMS spectrum'][
-                i]:  # checking if nan
-            for info in msdial_df['MSMS spectrum'][i].split():
-                mz, intensity = info.split(':')
-                peak = np.array([float(mz), float(intensity)])
-                peaks.append(peak)
-            new_spectrum = Spectrum(precursor_mz, peaks)
-            query_spectra.append(new_spectrum)
+    for i, row in msdial_df.iterrows():
+        new_spectrum = row_to_spectral_record(row, original_file=original_file)
+        query_spectra.append(new_spectrum)
     return query_spectra
 
 
+def row_to_spectral_record(row, original_file=None):
+    peaks = extract_msdial_spectrum(row, 'MSMS spectrum')
+    precursor_mz = row['Precursor m/z'].values[0]
+    metadata = {
+        'rt_in_minutes': row['RT (min)'].values[0],
+        'rt': row['RT (min)'].values[0] * 60.0,
+        'names': [row['Title'].values[0]],
+        'rt_matched': row['RT matched'].values[0],
+        'mz_matched': row['m/z matched'].values[0],
+        'msms_matched': row['MS/MS matched'].values[0],
+        'total_score': row['Total score'].values[0],
+        'dot_product': row['Dot product'].values[0],
+        'reverse_dot_product': row['Reverse dot product'].values[0],
+        'fragment_presence': row['Fragment presence %'].values[0],
+        'msms_spectrum': row['MSMS spectrum'].values[0],
+        'samples': {
+            'height': row['Height'].values[0],
+            'area': row['Area'].values[0]
+        },
+        'sample_count': 1,
+        'box': msdial_row_to_box(precursor_mz,
+                                 row["RT left(min)"].values[0],
+                                 row["RT right (min)"].values[0])
+    }
+    spectrum_id = 'peak_%.6f' % precursor_mz
+    new_spectrum = SpectralRecord(precursor_mz, peaks, metadata, original_file, spectrum_id)
+    return new_spectrum
+
+
+def msdial_row_to_box(precursor_mz, rt_left_in_minutes, rt_right_in_minutes, mz_tol=10):
+    x1 = rt_left_in_minutes * 60.0
+    x2 = rt_right_in_minutes * 60.0
+    y1 = precursor_mz * (1 - mz_tol / 1e6)
+    y2 = precursor_mz * (1 + mz_tol / 1e6)
+    return GenericBox(x1, x2, y1, y2)
+
+
 def make_queries_from_mzml(mzml_file_object):
     query_spectra = []
     for scan in mzml_file_object.scans:
@@ -138,18 +209,37 @@ def make_queries_from_chemicals(chemicals_file_name):
     chemicals = load_obj(chemicals_file_name)
     query_spectra = []
     for chem in chemicals:
-        precursor_mz = chem.isotopes[0][0]
-        peaks = []
-        for child in chem.children:
-            mz = child.isotopes[0][0]
-            intensity = child.parent.max_intensity * child.prop_ms2_mass
-            peak = np.array([mz, intensity])
-            peaks.append(peak)
-        new_spectrum = Spectrum(precursor_mz, peaks)
+        new_spectrum = chem_to_spectral_record(chem)
         query_spectra.append(new_spectrum)
     return query_spectra
 
 
+def chem_to_spectral_record(chem):
+    # FIXME: quick hack, assume that we observe M+H only
+    precursor_mz = chem.isotopes[0][0] + PROTON_MASS
+    peaks = []
+    for child in chem.children:
+        mz = child.isotopes[0][0]
+        intensity = child.parent.max_intensity * child.prop_ms2_mass
+        peak = np.array([mz, intensity])
+        peaks.append(peak)
+    metadata = { 'rt': chem.rt }
+    original_file = None
+    spectrum_id = 'peak_%.6f' % precursor_mz
+    new_spectrum = SpectralRecord(precursor_mz, peaks, metadata, original_file, spectrum_id)
+    return new_spectrum
+
+
+def scan_to_spectral_record(scan):
+    precursor_mz = scan.scan_params.get(ScanParameters.PRECURSOR_MZ)[0].precursor_mz
+    peaks = list(zip(scan.mzs, scan.intensities))
+    metadata = {}
+    original_file = None
+    spectrum_id = 'peak_%.6f' % precursor_mz
+    new_spectrum = SpectralRecord(precursor_mz, peaks, metadata, original_file, spectrum_id)
+    return new_spectrum
+
+
 def main(mzml_file_name, msp_file_name, precursor_tolerance, hit_threshold):
     mzml_file_objects = {}
     if os.path.isfile(mzml_file_name):
